@@ -34,7 +34,12 @@
 
 #include <string.h>
 #include <fcntl.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
+#ifdef G_OS_WIN32
+#include <io.h>
+#endif
 #include <libxml/xmlreader.h>
 #include <glib/gstdio.h>
 #include "gtksourceview-i18n.h"
@@ -65,6 +70,8 @@ struct _ParserState
 	GtkSourceLanguage *language;
 	GtkSourceContextData *ctx_data;
 
+	gchar *language_decoration;
+
 	/* A stack of id that representing parent contexts */
 	GQueue *curr_parents;
 
@@ -75,11 +82,15 @@ struct _ParserState
 	 * resolve references (the key is the id) */
 	GHashTable *defined_regexes;
 
-	/* The mapping between style ids and their default styles.
-	 * If lang file 'mama' contains this:
-	 * <style id="foo" map-to="def:blah"/>
-	 * <style id="bar"/>
-	 * then in styles_mapping: "mama:foo"->"def:blah", "mama:bar"->"mama:bar". */
+	/* Maps style ids to GtkSourceStyleInfo objects.
+	 * Contains all the styles defined in the lang files parsed
+	 * while parsing the main language file. For example, if the main
+	 * language is C, also styles in def.lang, gtk-doc.lang, etc. are
+	 * stored in this hash table. For styles defined in language files
+	 * different from the main one, the name is _not_ stored, since it
+	 * is not used in other places of the code. So, if the main language is
+	 * C, only the name of styles defined in c.lang is stored, while for
+	 * the styles defined in def.lang, etc. the name is not stored. */
 	GHashTable *styles_mapping;
 
 	/* The list of loaded languages (the item are xmlChar pointers),
@@ -98,9 +109,11 @@ struct _ParserState
 
 	gchar *opening_delimiter;
 	gchar *closing_delimiter;
-};
-typedef struct _ParserState ParserState;
 
+	GError *error;
+};
+
+typedef struct _ParserState ParserState;
 
 
 static GQuark     parser_error_quark           (void);
@@ -140,18 +153,13 @@ static gboolean   create_definition            (ParserState *parser_state,
                                                 gchar *style,
 						GError **error);
 
-static void       handle_context_element       (ParserState *parser_state,
-                                                GError **error);
-static void       handle_language_element      (ParserState *parser_state,
-                                                GError **error);
-static void       handle_define_regex_element  (ParserState *parser_state,
-                                                GError **error);
+static void       handle_context_element       (ParserState *parser_state);
+static void       handle_language_element      (ParserState *parser_state);
+static void       handle_define_regex_element  (ParserState *parser_state);
 static void       handle_default_regex_options_element
-                                               (ParserState *parser_state,
-                                                GError **error);
+                                               (ParserState *parser_state);
 static void       handle_replace_element       (ParserState *parser_state);
-static void       element_start                (ParserState *parser_state,
-						GError **error);
+static void       element_start                (ParserState *parser_state);
 static void       element_end                  (ParserState *parser_state);
 static gboolean   replace_by_id                (const GMatchInfo *match_info,
 						GString *expanded_regex,
@@ -638,8 +646,7 @@ create_sub_pattern (ParserState *parser_state,
 }
 
 static void
-handle_context_element (ParserState *parser_state,
-			GError **error)
+handle_context_element (ParserState *parser_state)
 {
 	gchar *id, *parent_id, *style_ref;
 	xmlChar *ref, *sub_pattern, *tmp;
@@ -650,11 +657,11 @@ handle_context_element (ParserState *parser_state,
 
 	GError *tmp_error = NULL;
 
-	g_return_if_fail (error != NULL && *error == NULL);
+	g_return_if_fail (parser_state->error == NULL);
 
 	ref = xmlTextReaderGetAttribute (parser_state->reader, BAD_CAST "ref");
 	sub_pattern = xmlTextReaderGetAttribute (parser_state->reader,
-			BAD_CAST "sub-pattern");
+						 BAD_CAST "sub-pattern");
 
 	tmp = xmlTextReaderGetAttribute (parser_state->reader, BAD_CAST "ignore-style");
 	if (tmp != NULL && str_to_bool (tmp))
@@ -670,7 +677,8 @@ handle_context_element (ParserState *parser_state,
 
 	if (ignore_style && ref == NULL)
 	{
-		g_set_error (error, PARSER_ERROR,
+		g_set_error (&parser_state->error,
+			     PARSER_ERROR,
 			     PARSER_ERROR_WRONG_STYLE,
 			     "ignore-style used not in a reference to context");
 
@@ -762,7 +770,7 @@ handle_context_element (ParserState *parser_state,
 					 * stack only if other contexts can be
 					 * defined inside it */
 					g_queue_push_head (parser_state->curr_parents,
-							g_strdup (id));
+							   g_strdup (id));
 				}
 			}
 		}
@@ -775,10 +783,7 @@ handle_context_element (ParserState *parser_state,
 	xmlFree (ref);
 
 	if (tmp_error != NULL)
-	{
-		g_propagate_error (error, tmp_error);
-		return;
-	}
+		g_propagate_error (&parser_state->error, tmp_error);
 }
 
 static void
@@ -805,33 +810,33 @@ handle_replace_element (ParserState *parser_state)
 }
 
 static void
-handle_language_element (ParserState *parser_state,
-			 GError **error)
+handle_language_element (ParserState *parser_state)
 {
 	/* FIXME: check that the language name, version, etc. are the
 	 * right ones - Paolo */
 	xmlChar *lang_id, *lang_version;
 	xmlChar *expected_version = BAD_CAST "2.0";
 
-	g_return_if_fail (error != NULL && *error == NULL);
+	g_return_if_fail (parser_state->error == NULL);
 
 	lang_version = xmlTextReaderGetAttribute (parser_state->reader, BAD_CAST "version");
 
 	if (lang_version == NULL ||
-			(xmlStrcmp (expected_version, lang_version) != 0))
+	    xmlStrcmp (expected_version, lang_version) != 0)
 	{
-		g_set_error (error,
+		g_set_error (&parser_state->error,
 			     PARSER_ERROR,
 			     PARSER_ERROR_WRONG_VERSION,
 			     "wrong language version '%s', expected '%s'",
 			     lang_version ? (gchar*) lang_version : "(none)",
 			     (gchar*) expected_version);
 	}
-
-	lang_id = xmlTextReaderGetAttribute (parser_state->reader, BAD_CAST "id");
-
-	parser_state->current_lang_id = g_strdup ((gchar *) lang_id);
-	g_hash_table_insert (parser_state->loaded_lang_ids, lang_id, lang_id);
+	else
+	{
+		lang_id = xmlTextReaderGetAttribute (parser_state->reader, BAD_CAST "id");
+		parser_state->current_lang_id = g_strdup ((gchar *) lang_id);
+		g_hash_table_insert (parser_state->loaded_lang_ids, lang_id, lang_id);
+	}
 
 	xmlFree (lang_version);
 }
@@ -962,7 +967,7 @@ expand_regex_vars (ParserState *parser_state, gchar *regex, gint len, GError **e
 	if (regex == NULL)
 		return NULL;
 
-	egg_re = g_regex_new (re, 0, 0, NULL);
+	egg_re = g_regex_new (re, G_REGEX_NEWLINE_LF, 0, NULL);
 
 	data.parser_state = parser_state;
 	data.error = NULL;
@@ -1042,12 +1047,13 @@ expand_regex_delimiters (ParserState *parser_state,
 	 */
 	const gchar *re = "(?<!\\\\)(\\\\\\\\)*\\\\%(\\[|\\])";
 	gchar *expanded_regex;
-	GRegex *egg_re;
+	static GRegex *egg_re;
 
 	if (regex == NULL)
 		return NULL;
 
-	egg_re = g_regex_new (re, 0, 0, NULL);
+	if (egg_re == NULL)
+		egg_re = g_regex_new (re, G_REGEX_NEWLINE_LF | G_REGEX_OPTIMIZE, 0, NULL);
 
 	expanded_regex = g_regex_replace_eval (egg_re, regex, len, 0, 0,
 					       replace_delimiter, parser_state, NULL);
@@ -1055,8 +1061,6 @@ expand_regex_delimiters (ParserState *parser_state,
 	DEBUG (g_message ("expanded regex delims '%s' to '%s'",
 				regex, expanded_regex));
 
-	g_regex_unref (egg_re);
-
 	return expanded_regex;
 }
 
@@ -1070,6 +1074,7 @@ expand_regex (ParserState *parser_state,
 {
 	gchar *tmp_regex;
 	GString *expanded_regex;
+	static GRegex *backref_re = NULL;
 
 	g_assert (parser_state != NULL);
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
@@ -1077,14 +1082,20 @@ expand_regex (ParserState *parser_state,
 	if (regex == NULL)
 		return NULL;
 
-	if (g_regex_match_simple ("(?<!\\\\)(\\\\\\\\)*\\\\[0-9]", regex, 0, 0))
+	if (backref_re == NULL)
+		backref_re = g_regex_new ("(?<!\\\\)(\\\\\\\\)*\\\\[0-9]",
+					  G_REGEX_OPTIMIZE | G_REGEX_NEWLINE_LF,
+					  0,
+					  NULL);
+
+	if (g_regex_match (backref_re, regex, 0, NULL))
 	{
 		/* This may be a backreference, or it may be an octal character */
 		GRegex *compiled;
 
-		compiled = g_regex_new (regex, flags, 0, error);
+		compiled = g_regex_new (regex, flags | G_REGEX_NEWLINE_LF, 0, error);
 
-		if (!compiled)
+		if (compiled == NULL)
 			return NULL;
 
 		if (g_regex_get_max_backref (compiled) > 0)
@@ -1163,8 +1174,7 @@ expand_regex (ParserState *parser_state,
 }
 
 static void
-handle_define_regex_element (ParserState *parser_state,
-			     GError **error)
+handle_define_regex_element (ParserState *parser_state)
 {
 	gchar *id;
 	xmlChar *regex;
@@ -1177,7 +1187,7 @@ handle_define_regex_element (ParserState *parser_state,
 
 	int type;
 
-	g_return_if_fail (error != NULL && *error == NULL);
+	g_return_if_fail (parser_state->error == NULL);
 
 	if (parser_state->ctx_data == NULL)
 		return;
@@ -1230,49 +1240,43 @@ handle_define_regex_element (ParserState *parser_state,
 	xmlFree (regex);
 
 	if (tmp_error != NULL)
-	{
-		g_propagate_error (error, tmp_error);
-		return;
-	}
+		g_propagate_error (&parser_state->error, tmp_error);
 }
 
 static void
-handle_default_regex_options_element (ParserState *parser_state,
-				      GError     **error)
+handle_default_regex_options_element (ParserState *parser_state)
 {
 	xmlNode *elm;
 
-	g_return_if_fail (error != NULL && *error == NULL);
+	g_return_if_fail (parser_state->error == NULL);
 
 	if (parser_state->ctx_data == NULL)
 		return;
 
-	elm = xmlTextReaderCurrentNode	(parser_state->reader);
+	elm = xmlTextReaderCurrentNode (parser_state->reader);
 
 	parser_state->regex_compile_flags = get_regex_flags (elm, 0);
 }
 
 static void
 parse_language_with_id (ParserState *parser_state,
-		gchar *lang_id,
-		GError **error)
+			gchar       *lang_id)
 {
 	GtkSourceLanguageManager *lm;
 	GtkSourceLanguage *imported_language;
-	GError *tmp_error = NULL;
 
-	g_return_if_fail (error != NULL && *error == NULL);
+	g_return_if_fail (parser_state->error == NULL);
 
 	lm = _gtk_source_language_get_language_manager (parser_state->language);
 	imported_language = gtk_source_language_manager_get_language_by_id (lm, lang_id);
 
 	if (imported_language == NULL)
 	{
-		g_set_error (&tmp_error,
-				PARSER_ERROR,
-				PARSER_ERROR_WRONG_ID,
-				"unable to resolve language '%s'",
-				lang_id);
+		g_set_error (&parser_state->error,
+			     PARSER_ERROR,
+			     PARSER_ERROR_WRONG_ID,
+			     "unable to resolve language '%s'",
+			     lang_id);
 	}
 	else
 	{
@@ -1283,27 +1287,19 @@ parse_language_with_id (ParserState *parser_state,
 			    parser_state->styles_mapping,
 			    parser_state->loaded_lang_ids,
 			    parser_state->replacements,
-			    &tmp_error);
-	}
-
-	if (tmp_error != NULL)
-	{
-		g_propagate_error (error, tmp_error);
-		return;
+			    &parser_state->error);
 	}
 }
 
 static void
-parse_style (ParserState *parser_state,
-	     GError     **error)
+parse_style (ParserState *parser_state)
 {
 	gchar *id;
 	xmlChar *name, *map_to;
 	xmlChar *tmp;
 	gchar *lang_id = NULL;
-	GError *tmp_error = NULL;
 
-	g_return_if_fail (error != NULL && *error == NULL);
+	g_return_if_fail (parser_state->error == NULL);
 
 	tmp = xmlTextReaderGetAttribute (parser_state->reader,
 					 BAD_CAST "id");
@@ -1318,7 +1314,6 @@ parse_style (ParserState *parser_state,
 	name = xmlTextReaderGetAttribute (parser_state->reader,
 					  BAD_CAST "_name");
 
-	/* FIXME: actually use this name somehow */
 	if (name != NULL)
 	{
 		gchar *tmp2 = _gtk_source_language_translate_string (parser_state->language,
@@ -1339,49 +1334,58 @@ parse_style (ParserState *parser_state,
 
 	if (map_to != NULL && !id_is_decorated ((gchar*) map_to, &lang_id))
 	{
-		g_set_error (&tmp_error,
+		g_set_error (&parser_state->error,
 			     PARSER_ERROR,
 			     PARSER_ERROR_MALFORMED_MAP_TO,
 			     "the map-to attribute '%s' for the style '%s' lacks the prefix",
 			     map_to, id);
 	}
 
-	if (tmp_error == NULL && lang_id != NULL && lang_id[0] == 0)
+	if (parser_state->error == NULL && lang_id != NULL && lang_id[0] == 0)
 	{
 		g_free (lang_id);
 		lang_id = NULL;
 	}
 
-	if (tmp_error == NULL && lang_id != NULL && !lang_id_is_already_loaded (parser_state, lang_id))
-		parse_language_with_id (parser_state, lang_id, &tmp_error);
+	if (parser_state->error == NULL && lang_id != NULL &&
+	    !lang_id_is_already_loaded (parser_state, lang_id))
+	{
+		parse_language_with_id (parser_state, lang_id);
+	}
 
 	DEBUG (g_message ("style %s (%s) to be mapped to '%s'",
 			  name, id, map_to ? (char*) map_to : "(null)"));
 
-	if (tmp_error == NULL)
-		g_hash_table_insert (parser_state->styles_mapping, g_strdup (id),
-				     map_to ? g_strdup ((char*) map_to) : g_strdup (id));
+	if (parser_state->error == NULL)
+	{
+
+		GtkSourceStyleInfo *info;
+
+		/* Remember the style name only if the style has been defined in
+		 * the lang file we are parsing */
+		if (g_str_has_prefix (id, parser_state->language_decoration))
+			info = _gtk_source_style_info_new ((gchar *) name,
+							   (gchar *) map_to);
+		else
+			info = _gtk_source_style_info_new (NULL,
+							   (gchar *) map_to);
+
+		g_hash_table_insert (parser_state->styles_mapping, g_strdup (id), info);
+	}
 
 	g_free (lang_id);
 	g_free (id);
 	xmlFree (name);
 	xmlFree (map_to);
-
-	if (tmp_error != NULL)
-	{
-		g_propagate_error (error, tmp_error);
-		return;
-	}
 }
 
 static void
-handle_keyword_char_class_element (ParserState *parser_state,
-				   GError     **error)
+handle_keyword_char_class_element (ParserState *parser_state)
 {
 	xmlChar *char_class;
 	int ret, type;
 
-	g_return_if_fail (error != NULL && *error == NULL);
+	g_return_if_fail (parser_state->error == NULL);
 
 	if (parser_state->ctx_data == NULL)
 		return;
@@ -1407,16 +1411,14 @@ handle_keyword_char_class_element (ParserState *parser_state,
 }
 
 static void
-handle_styles_element (ParserState *parser_state,
-		       GError     **error)
+handle_styles_element (ParserState *parser_state)
 {
 	int ret, type;
 	const xmlChar *tag_name;
-	GError *tmp_error = NULL;
 
-	g_return_if_fail (error != NULL && *error == NULL);
+	g_return_if_fail (parser_state->error == NULL);
 
-	while (TRUE)
+	while (parser_state->error == NULL)
 	{
 		ret = xmlTextReaderRead (parser_state->reader);
 
@@ -1424,7 +1426,7 @@ handle_styles_element (ParserState *parser_state,
 
 		/* FIXME: is xmlTextReaderIsValid call needed here or
 		 * error func will be called? */
-		if (*error != NULL)
+		if (parser_state->error != NULL)
 			break;
 
 		tag_name = xmlTextReaderConstName (parser_state->reader);
@@ -1440,64 +1442,35 @@ handle_styles_element (ParserState *parser_state,
 			continue;
 
 		/* Handle <style> elements */
-		parse_style (parser_state, &tmp_error);
-
-		if (tmp_error != NULL)
-			break;
-	}
-
-	if (tmp_error != NULL)
-	{
-		g_propagate_error (error, tmp_error);
-		return;
+		parse_style (parser_state);
 	}
 }
 
 
 static void
-element_start (ParserState *parser_state,
-	       GError     **error)
+element_start (ParserState *parser_state)
 {
 	const xmlChar *name;
-	GError *tmp_error = NULL;
+
+	g_return_if_fail (parser_state->error == NULL);
 
 	/* TODO: check the namespace and ignore everithing is not in our namespace */
 	name = xmlTextReaderConstName (parser_state->reader);
 
 	if (xmlStrcmp (BAD_CAST "context", name) == 0)
-	{
-		handle_context_element (parser_state, &tmp_error);
-	}
+		handle_context_element (parser_state);
 	else if (xmlStrcmp (BAD_CAST "replace", name) == 0)
-	{
 		handle_replace_element (parser_state);
-	}
 	else if (xmlStrcmp (BAD_CAST "define-regex", name) == 0)
-	{
-		handle_define_regex_element (parser_state, &tmp_error);
-	}
+		handle_define_regex_element (parser_state);
 	else if (xmlStrcmp (BAD_CAST "language", name) == 0)
-	{
-		handle_language_element (parser_state, &tmp_error);
-	}
+		handle_language_element (parser_state);
 	else if (xmlStrcmp (BAD_CAST "styles", name) == 0)
-	{
-		handle_styles_element (parser_state, &tmp_error);
-	}
+		handle_styles_element (parser_state);
 	else if (xmlStrcmp (BAD_CAST "keyword-char-class", name) == 0)
-	{
-		handle_keyword_char_class_element (parser_state, &tmp_error);
-	}
+		handle_keyword_char_class_element (parser_state);
 	else if (xmlStrcmp (BAD_CAST "default-regex-options", name) == 0)
-	{
-		handle_default_regex_options_element (parser_state, &tmp_error);
-	}
-
-	if (tmp_error != NULL)
-	{
-		g_propagate_error (error, tmp_error);
-		return;
-	}
+		handle_default_regex_options_element (parser_state);
 }
 
 static void
@@ -1517,16 +1490,22 @@ element_end (ParserState *parser_state)
 }
 
 static void
-text_reader_structured_error_func (GError     **gerror,
+text_reader_structured_error_func (ParserState *parser_state,
 				   xmlErrorPtr  error)
 {
 	/* FIXME: does someone know how to use libxml api? */
 
-	if (*gerror == NULL)
-		g_set_error (gerror, PARSER_ERROR, PARSER_ERROR_INVALID_DOC, "parser error");
-
-	/* XXX: g_print now because of --g-fatal-warnings */
-	g_print ("in file %s on line %d: %s\n", error->file, error->line, error->message);
+	if (parser_state->error == NULL)
+		g_set_error (&parser_state->error,
+			     PARSER_ERROR,
+			     PARSER_ERROR_INVALID_DOC,
+			     "in file %s on line %d: %s\n",
+			     error->file,
+			     error->line,
+			     error->message);
+	else
+		/* extra warning won't hurt and it could give a clue about what's wrong */
+		g_warning ("in file %s on line %d: %s\n", error->file, error->line, error->message);
 }
 
 static gboolean
@@ -1596,17 +1575,17 @@ file_parse (gchar                     *filename,
 					 loaded_lang_ids);
 	xmlTextReaderSetStructuredErrorHandler (reader,
 						(xmlStructuredErrorFunc) text_reader_structured_error_func,
-						&tmp_error);
+						parser_state);
 
-	while (!tmp_error && (ret = xmlTextReaderRead (parser_state->reader)) == 1)
+	while (parser_state->error == NULL &&
+	       (ret = xmlTextReaderRead (parser_state->reader)) == 1)
 	{
 		int type;
 
-		xmlTextReaderIsValid (reader);
+		/* FIXME: does xmlTextReaderRead already do it? */
+		xmlTextReaderIsValid (parser_state->reader);
 
-		/* FIXME: is xmlTextReaderIsValid call needed here or
-		 * error func will be called? */
-		if (tmp_error != NULL)
+		if (parser_state->error != NULL)
 			break;
 
 		type = xmlTextReaderNodeType (parser_state->reader);
@@ -1614,7 +1593,7 @@ file_parse (gchar                     *filename,
 		switch (type)
 		{
 			case XML_READER_TYPE_ELEMENT:
-				element_start (parser_state, &tmp_error);
+				element_start (parser_state);
 				break;
 			case XML_READER_TYPE_END_ELEMENT:
 				element_end (parser_state);
@@ -1622,6 +1601,12 @@ file_parse (gchar                     *filename,
 		}
 	}
 
+	if (parser_state->error != NULL)
+	{
+		g_propagate_error (&tmp_error, parser_state->error);
+		parser_state->error = NULL;
+	}
+
 	parser_state_destroy (parser_state);
 
 	if (tmp_error != NULL)
@@ -1648,17 +1633,21 @@ parser_state_new (GtkSourceLanguage       *language,
 		  GHashTable              *loaded_lang_ids)
 {
 	ParserState *parser_state;
-	parser_state = g_slice_new (ParserState);
+	parser_state = g_slice_new0 (ParserState);
 
 	parser_state->language = language;
 	parser_state->ctx_data = ctx_data;
 
+	g_return_val_if_fail (language->priv->id != NULL, NULL);
+	parser_state->language_decoration = g_strdup_printf ("%s:", language->priv->id);
+
 	parser_state->current_lang_id = NULL;
 
 	parser_state->id_cookie = 0;
 	parser_state->regex_compile_flags = 0;
 
 	parser_state->reader = reader;
+	parser_state->error = NULL;
 
 	parser_state->defined_regexes = defined_regexes;
 	parser_state->styles_mapping = styles_mapping;
@@ -1680,27 +1669,27 @@ parser_state_destroy (ParserState *parser_state)
 	if (parser_state->reader != NULL)
 		xmlFreeTextReader (parser_state->reader);
 
+	if (parser_state->error != NULL)
+		g_error_free (parser_state->error);
+
 	g_queue_free (parser_state->curr_parents);
 	g_free (parser_state->current_lang_id);
 
 	g_free (parser_state->opening_delimiter);
 	g_free (parser_state->closing_delimiter);
 
+	g_free (parser_state->language_decoration);
+
 	g_slice_free (ParserState, parser_state);
 }
 
 static gboolean
-steal_styles_mapping (char       *style,
-		      char       *map_to,
-		      GHashTable *styles)
+steal_styles_mapping (gchar              *style_id,
+		      GtkSourceStyleInfo *info,
+		      GHashTable         *styles)
 {
-	if (strcmp (style, map_to) != 0)
-	{
-		g_hash_table_insert (styles, style, map_to);
-		return TRUE;
-	}
-
-	return FALSE;
+	g_hash_table_insert (styles, style_id, info);
+	return TRUE;
 }
 
 gboolean
@@ -1729,8 +1718,10 @@ _gtk_source_language_file_parse_version2 (GtkSourceLanguage       *language,
 
 	defined_regexes = g_hash_table_new_full (g_str_hash, g_str_equal,
 						 g_free, g_free);
-	styles = g_hash_table_new_full (g_str_hash, g_str_equal,
-					g_free, g_free);
+	styles = g_hash_table_new_full (g_str_hash,
+					g_str_equal,
+					g_free,
+					(GDestroyNotify) _gtk_source_style_info_free);
 	loaded_lang_ids = g_hash_table_new_full (g_str_hash, g_str_equal,
 						 (GDestroyNotify) xmlFree,
 						 NULL);
