@@ -59,9 +59,11 @@ GtkSourceStyleScheme	*_gtk_source_style_scheme_new		(const gchar          *id,
 
 const gchar             *gtk_source_style_scheme_get_id         (GtkSourceStyleScheme *scheme);
 const gchar             *gtk_source_style_scheme_get_name       (GtkSourceStyleScheme *scheme);
+const gchar             *gtk_source_style_scheme_get_description(GtkSourceStyleScheme *scheme);
+const gchar             *gtk_source_style_scheme_get_filename	(GtkSourceStyleScheme *scheme);
 
 GtkSourceStyle		*gtk_source_style_scheme_get_style	(GtkSourceStyleScheme *scheme,
-								 const gchar          *style_name);
+								 const gchar          *style_id);
 GtkSourceStyle		*gtk_source_style_scheme_get_matching_brackets_style
 								(GtkSourceStyleScheme *scheme);
 gboolean		 gtk_source_style_scheme_get_current_line_color
